@@ -1,7 +1,10 @@
 package integration
 
 import (
+	"net/netip"
+
 	v1 "github.com/juanfont/headscale/gen/go/headscale/v1"
+	policyv1 "github.com/juanfont/headscale/hscontrol/policy/v1"
 	"github.com/ory/dockertest/v3"
 )
 
@@ -15,11 +18,15 @@ type ControlServer interface {
 	GetHealthEndpoint() string
 	GetEndpoint() string
 	WaitForRunning() error
-	CreateUser(user string) error
-	CreateAuthKey(user string, reusable bool, ephemeral bool) (*v1.PreAuthKey, error)
+	CreateUser(user string) (*v1.User, error)
+	CreateAuthKey(user uint64, reusable bool, ephemeral bool) (*v1.PreAuthKey, error)
 	ListNodes(users ...string) ([]*v1.Node, error)
+	NodesByUser() (map[string][]*v1.Node, error)
+	NodesByName() (map[string]*v1.Node, error)
 	ListUsers() ([]*v1.User, error)
+	MapUsers() (map[string]*v1.User, error)
+	ApproveRoutes(uint64, []netip.Prefix) (*v1.Node, error)
 	GetCert() []byte
 	GetHostname() string
-	GetIP() string
+	SetPolicy(*policyv1.ACLPolicy) error
 }
