@@ -2,9 +2,11 @@ package dockertestutil
 
 import (
 	"errors"
-	"net"
 	"fmt"
+	"log"
+	"net"
 
+	"github.com/juanfont/headscale/hscontrol/util"
 	"github.com/ory/dockertest/v3"
 	"github.com/ory/dockertest/v3/docker"
 )
@@ -84,3 +86,43 @@ func RandomFreeHostPort() (int, error) {
 	//nolint:forcetypeassert
 	return listener.Addr().(*net.TCPAddr).Port, nil
 }
+
+// CleanUnreferencedNetworks removes networks that are not referenced by any containers.
+func CleanUnreferencedNetworks(pool *dockertest.Pool) error {
+	filter := "name=hs-"
+	networks, err := pool.NetworksByName(filter)
+	if err != nil {
+		return fmt.Errorf("getting networks by filter %q: %w", filter, err)
+	}
+
+	for _, network := range networks {
+		if network.Network.Containers == nil || len(network.Network.Containers) == 0 {
+			err := pool.RemoveNetwork(&network)
+			if err != nil {
+				log.Printf("removing network %s: %s", network.Network.Name, err)
+			}
+		}
+	}
+
+	return nil
+}
+
+// CleanImagesInCI removes images if running in CI.
+func CleanImagesInCI(pool *dockertest.Pool) error {
+	if !util.IsCI() {
+		log.Println("Skipping image cleanup outside of CI")
+		return nil
+	}
+
+	images, err := pool.Client.ListImages(docker.ListImagesOptions{})
+	if err != nil {
+		return fmt.Errorf("getting images: %w", err)
+	}
+
+	for _, image := range images {
+		log.Printf("removing image: %s, %v", image.ID, image.RepoTags)
+		_ = pool.Client.RemoveImage(image.ID)
+	}
+
+	return nil
+}
